@@ -219,102 +219,120 @@ u_char	*esc_spt	= (u_char *)0;
  * state.  The states are ordered: Normal, SHIFT, CTRL, ALT,
  * SHIFT/ALT.
  */
+
+/* This new keymap from Tudor Hulubei (tudor@cs.unh.edu) makes the
+   following changes to the keyboard driver:
+
+   - Alt + key (m-key) returns `ESC key' instead of `ESC N key'.
+   - Backspace returns 0x7f instead of 0x08.
+   - Delete returns `ESC [ 9' instead of 0x7f.
+   - Alt + function keys return key sequences that are different
+     from the key sequences returned by the function keys alone.
+     This is done with the idea of alowing a terminal server to
+     implement multiple virtual consoles mapped on Alt+F1, Alt+F2,
+     etc, as in Linux.
+
+   -- Derek Upham 1997/06/25 */
+
 unsigned char	key_map[NUMKEYS][WIDTH_KMAP] = {
-{NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC},
-{K_ESC,NC,NC, K_ESC,NC,NC, K_ESC,NC,NC, K_ESC,NC,NC, K_ESC,NC,NC},
-{K_ONE,NC,NC, K_BANG,NC,NC, K_ONE,NC,NC, 0x1b,0x4e,0x31, 0x1b,0x4e,0x21},
-{K_TWO,NC,NC, K_ATSN,NC,NC, K_NUL,NC,NC, 0x1b,0x4e,0x32, 0x1b,0x4e,0x40},
-{K_THREE,NC,NC, K_POUND,NC,NC, K_THREE,NC,NC, 0x1b,0x4e,0x33, 0x1b,0x4e,0x23},
-{K_FOUR,NC,NC, K_DOLLAR,NC,NC, K_FOUR,NC,NC, 0x1b,0x4e,0x34, 0x1b,0x4e,0x24},
-{K_FIVE,NC,NC, K_PERC,NC,NC, K_FIVE,NC,NC, 0x1b,0x4e,0x35, 0x1b,0x4e,0x25},
-{K_SIX,NC,NC, K_CARET,NC,NC, K_RS,NC,NC, 0x1b,0x4e,0x36, 0x1b,0x4e,0x5e},
-{K_SEVEN,NC,NC, K_AMPER,NC,NC, K_SEVEN,NC,NC, 0x1b,0x4e,0x37, 0x1b,0x4e,0x26},
-{K_EIGHT,NC,NC, K_ASTER,NC,NC, K_EIGHT,NC,NC, 0x1b,0x4e,0x38, 0x1b,0x4e,0x2a},
-{K_NINE,NC,NC, K_LPAREN,NC,NC, K_NINE,NC,NC, 0x1b,0x4e,0x39,0x1b,0x4e,0x28},
-{K_ZERO,NC,NC, K_RPAREN,NC,NC, K_ZERO,NC,NC, 0x1b,0x4e,0x30,0x1b,0x4e,0x29},
-{K_MINUS,NC,NC, K_UNDSC,NC,NC, K_US,NC,NC, 0x1b,0x4e,0x2d, 0x1b,0x4e,0x5f},
-{K_EQL,NC,NC, K_PLUS,NC,NC, K_EQL,NC,NC, 0x1b,0x4e,0x3d, 0x1b,0x4e,0x2b},
-{K_BS,NC,NC, K_BS,NC,NC, K_BS,NC,NC, K_BS,NC,NC, K_BS,NC,NC},
-{K_HT,NC,NC, K_GS,NC,NC, K_HT,NC,NC, K_HT,NC,NC, K_GS,NC,NC},
-{K_q,NC,NC, K_Q,NC,NC, K_DC1,NC,NC, 0x1b,0x4e,0x71, 0x1b,0x4e,0x51},
-{K_w,NC,NC, K_W,NC,NC, K_ETB,NC,NC, 0x1b,0x4e,0x77, 0x1b,0x4e,0x57},
-{K_e,NC,NC, K_E,NC,NC, K_ENQ,NC,NC, 0x1b,0x4e,0x65, 0x1b,0x4e,0x45},
-{K_r,NC,NC, K_R,NC,NC, K_DC2,NC,NC, 0x1b,0x4e,0x72, 0x1b,0x4e,0x52},
-{K_t,NC,NC, K_T,NC,NC, K_DC4,NC,NC, 0x1b,0x4e,0x74, 0x1b,0x4e,0x54},
-{K_y,NC,NC, K_Y,NC,NC, K_EM,NC,NC, 0x1b,0x4e,0x79, 0x1b,0x4e,0x59},
-{K_u,NC,NC, K_U,NC,NC, K_NAK,NC,NC, 0x1b,0x4e,0x75, 0x1b,0x4e,0x55},
-{K_i,NC,NC, K_I,NC,NC, K_HT,NC,NC, 0x1b,0x4e,0x69, 0x1b,0x4e,0x49},
-{K_o,NC,NC, K_O,NC,NC, K_SI,NC,NC, 0x1b,0x4e,0x6f, 0x1b,0x4e,0x4f},
-{K_p,NC,NC, K_P,NC,NC, K_DLE,NC,NC, 0x1b,0x4e,0x70, 0x1b,0x4e,0x50},
-{K_LBRKT,NC,NC, K_LBRACE,NC,NC, K_ESC,NC,NC, 0x1b,0x4e,0x5b, 0x1b,0x4e,0x7b},
-{K_RBRKT,NC,NC, K_RBRACE,NC,NC, K_GS,NC,NC, 0x1b,0x4e,0x5d, 0x1b,0x4e,0x7d},
-{K_CR,NC,NC, K_CR,NC,NC, K_CR,NC,NC, K_CR,NC,NC, K_CR,NC,NC},
-{K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC, 
-	 K_SCAN,K_CTLSC,NC},
-{K_a,NC,NC, K_A,NC,NC, K_SOH,NC,NC, 0x1b,0x4e,0x61, 0x1b,0x4e,0x41},
-{K_s,NC,NC, K_S,NC,NC, K_DC3,NC,NC, 0x1b,0x4e,0x73, 0x1b,0x4e,0x53},
-{K_d,NC,NC, K_D,NC,NC, K_EOT,NC,NC, 0x1b,0x4e,0x65, 0x1b,0x4e,0x45},
-{K_f,NC,NC, K_F,NC,NC, K_ACK,NC,NC, 0x1b,0x4e,0x66, 0x1b,0x4e,0x46},
-{K_g,NC,NC, K_G,NC,NC, K_BEL,NC,NC, 0x1b,0x4e,0x67, 0x1b,0x4e,0x47},
-{K_h,NC,NC, K_H,NC,NC, K_BS,NC,NC, 0x1b,0x4e,0x68, 0x1b,0x4e,0x48},
-{K_j,NC,NC, K_J,NC,NC, K_LF,NC,NC, 0x1b,0x4e,0x6a, 0x1b,0x4e,0x4a},
-{K_k,NC,NC, K_K,NC,NC, K_VT,NC,NC, 0x1b,0x4e,0x6b, 0x1b,0x4e,0x4b},
-{K_l,NC,NC, K_L,NC,NC, K_FF,NC,NC, 0x1b,0x4e,0x6c, 0x1b,0x4e,0x4c},
-{K_SEMI,NC,NC, K_COLON,NC,NC, K_SEMI,NC,NC, 0x1b,0x4e,0x3b, 0x1b,0x4e,0x3a},
-{K_SQUOTE,NC,NC,K_DQUOTE,NC,NC,K_SQUOTE,NC,NC,0x1b,0x4e,0x27,0x1b,0x4e,0x22},
-{K_GRAV,NC,NC, K_TILDE,NC,NC, K_RS,NC,NC, 0x1b,0x4e,0x60, 0x1b,0x4e,0x7e},
+{NC,NC,NC,      NC,NC,NC,       NC,NC,NC,      NC,NC,NC,      NC,NC,NC},
+{K_ESC,NC,NC,   K_ESC,NC,NC,    K_ESC,NC,NC,   0x1b,K_ESC,NC, K_ESC,NC,NC},
+{K_ONE,NC,NC,   K_BANG,NC,NC,   K_ONE,NC,NC,   0x1b,0x31,NC,  0x1b,0x4e,0x21},
+{K_TWO,NC,NC,   K_ATSN,NC,NC,   K_NUL,NC,NC,   0x1b,0x32,NC,  0x1b,0x4e,0x40},
+{K_THREE,NC,NC, K_POUND,NC,NC,  K_THREE,NC,NC, 0x1b,0x33,NC,  0x1b,0x4e,0x23},
+{K_FOUR,NC,NC,  K_DOLLAR,NC,NC, K_FOUR,NC,NC,  0x1b,0x34,NC,  0x1b,0x4e,0x24},
+{K_FIVE,NC,NC,  K_PERC,NC,NC,   K_FIVE,NC,NC,  0x1b,0x35,NC,  0x1b,0x4e,0x25},
+{K_SIX,NC,NC,   K_CARET,NC,NC,  K_RS,NC,NC,    0x1b,0x36,NC,  0x1b,0x4e,0x5e},
+{K_SEVEN,NC,NC, K_AMPER,NC,NC,  K_SEVEN,NC,NC, 0x1b,0x37,NC,  0x1b,0x4e,0x26},
+{K_EIGHT,NC,NC, K_ASTER,NC,NC,  K_EIGHT,NC,NC, 0x1b,0x38,NC,  0x1b,0x4e,0x2a},
+{K_NINE,NC,NC,  K_LPAREN,NC,NC, K_NINE,NC,NC,  0x1b,0x39,NC,  0x1b,0x4e,0x28},
+{K_ZERO,NC,NC,  K_RPAREN,NC,NC, K_ZERO,NC,NC,  0x1b,0x30,NC,  0x1b,0x4e,0x29},
+{K_MINUS,NC,NC, K_UNDSC,NC,NC,  K_US,NC,NC,    0x1b,0x2d,NC,  0x1b,0x4e,0x5f},
+{K_EQL,NC,NC,   K_PLUS,NC,NC,   K_EQL,NC,NC,   0x1b,0x3d,NC,  0x1b,0x4e,0x2b},
+{K_DEL,NC,NC,   K_DEL,NC,NC,    K_DEL,NC,NC,   0x1b,K_DEL,NC, K_DEL,NC,NC},
+{K_HT,NC,NC,    K_GS,NC,NC,     K_HT,NC,NC,    0x1b,K_HT,NC,  K_GS,NC,NC},
+{K_q,NC,NC,     K_Q,NC,NC,      K_DC1,NC,NC,   0x1b,0x71,NC,  0x1b,0x4e,0x51},
+{K_w,NC,NC,     K_W,NC,NC,      K_ETB,NC,NC,   0x1b,0x77,NC,  0x1b,0x4e,0x57},
+{K_e,NC,NC,     K_E,NC,NC,      K_ENQ,NC,NC,   0x1b,0x65,NC,  0x1b,0x4e,0x45},
+{K_r,NC,NC,     K_R,NC,NC,      K_DC2,NC,NC,   0x1b,0x72,NC,  0x1b,0x4e,0x52},
+{K_t,NC,NC,     K_T,NC,NC,      K_DC4,NC,NC,   0x1b,0x74,NC,  0x1b,0x4e,0x54},
+{K_y,NC,NC,     K_Y,NC,NC,      K_EM,NC,NC,    0x1b,0x79,NC,  0x1b,0x4e,0x59},
+{K_u,NC,NC,     K_U,NC,NC,      K_NAK,NC,NC,   0x1b,0x75,NC,  0x1b,0x4e,0x55},
+{K_i,NC,NC,     K_I,NC,NC,      K_HT,NC,NC,    0x1b,0x69,NC,  0x1b,0x4e,0x49},
+{K_o,NC,NC,     K_O,NC,NC,      K_SI,NC,NC,    0x1b,0x6f,NC,  0x1b,0x4e,0x4f},
+{K_p,NC,NC,     K_P,NC,NC,      K_DLE,NC,NC,   0x1b,0x70,NC,  0x1b,0x4e,0x50},
+{K_LBRKT,NC,NC, K_LBRACE,NC,NC, K_ESC,NC,NC,   0x1b,0x5b,NC,  0x1b,0x4e,0x7b},
+{K_RBRKT,NC,NC, K_RBRACE,NC,NC, K_GS,NC,NC,    0x1b,0x5d,NC,  0x1b,0x4e,0x7d},
+{K_CR,NC,NC,    K_CR,NC,NC,     K_CR,NC,NC,    0x1b,K_CR,NC,  K_CR,NC,NC},
+{K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC, K_SCAN,K_CTLSC,NC,
+     K_SCAN,K_CTLSC,NC},
+{K_a,NC,NC,     K_A,NC,NC,      K_SOH,NC,NC,   0x1b,0x61,NC,  0x1b,0x4e,0x41},
+{K_s,NC,NC,     K_S,NC,NC,      K_DC3,NC,NC,   0x1b,0x73,NC,  0x1b,0x4e,0x53},
+{K_d,NC,NC,     K_D,NC,NC,      K_EOT,NC,NC,   0x1b,0x64,NC,  0x1b,0x4e,0x45},
+{K_f,NC,NC,     K_F,NC,NC,      K_ACK,NC,NC,   0x1b,0x66,NC,  0x1b,0x4e,0x46},
+{K_g,NC,NC,     K_G,NC,NC,      K_BEL,NC,NC,   0x1b,0x67,NC,  0x1b,0x4e,0x47},
+{K_h,NC,NC,     K_H,NC,NC,      K_BS,NC,NC,    0x1b,0x68,NC,  0x1b,0x4e,0x48},
+{K_j,NC,NC,     K_J,NC,NC,      K_LF,NC,NC,    0x1b,0x6a,NC,  0x1b,0x4e,0x4a},
+{K_k,NC,NC,     K_K,NC,NC,      K_VT,NC,NC,    0x1b,0x6b,NC,  0x1b,0x4e,0x4b},
+{K_l,NC,NC,     K_L,NC,NC,      K_FF,NC,NC,    0x1b,0x6c,NC,  0x1b,0x4e,0x4c},
+{K_SEMI,NC,NC,  K_COLON,NC,NC,  K_SEMI,NC,NC,  0x1b,0x3b,NC,  0x1b,0x4e,0x3a},
+{K_SQUOTE,NC,NC,K_DQUOTE,NC,NC, K_SQUOTE,NC,NC,0x1b,0x27,NC,  0x1b,0x4e,0x22},
+{K_GRAV,NC,NC,  K_TILDE,NC,NC,  K_RS,NC,NC,    0x1b,0x60,NC,  0x1b,0x4e,0x7e},
 {K_SCAN,K_LSHSC,NC, K_SCAN,K_LSHSC,NC, K_SCAN,K_LSHSC,NC, K_SCAN,K_LSHSC,NC,
-	 K_SCAN,K_LSHSC,NC},
-{K_BSLSH,NC,NC, K_PIPE,NC,NC, K_FS,NC,NC, 0x1b,0x4e,0x5c, 0x1b,0x4e,0x7c},
-{K_z,NC,NC, K_Z,NC,NC, K_SUB,NC,NC, 0x1b,0x4e,0x7a, 0x1b,0x4e,0x5a},
-{K_x,NC,NC, K_X,NC,NC, K_CAN,NC,NC, 0x1b,0x4e,0x78, 0x1b,0x4e,0x58},
-{K_c,NC,NC, K_C,NC,NC, K_ETX,NC,NC, 0x1b,0x4e,0x63, 0x1b,0x4e,0x43},
-{K_v,NC,NC, K_V,NC,NC, K_SYN,NC,NC, 0x1b,0x4e,0x76, 0x1b,0x4e,0x56},
-{K_b,NC,NC, K_B,NC,NC, K_STX,NC,NC, 0x1b,0x4e,0x62, 0x1b,0x4e,0x42},
-{K_n,NC,NC, K_N,NC,NC, K_SO,NC,NC, 0x1b,0x4e,0x6e, 0x1b,0x4e,0x4e},
-{K_m,NC,NC, K_M,NC,NC, K_CR,NC,NC, 0x1b,0x4e,0x6d, 0x1b,0x4e,0x4d},
-{K_COMMA,NC,NC, K_LTHN,NC,NC, K_COMMA,NC,NC, 0x1b,0x4e,0x2c, 0x1b,0x4e,0x3c},
-{K_PERIOD,NC,NC, K_GTHN,NC,NC, K_PERIOD,NC,NC,0x1b,0x4e,0x2e,0x1b,0x4e,0x3e},
-{K_SLASH,NC,NC, K_QUES,NC,NC, K_SLASH,NC,NC, 0x1b,0x4e,0x2f, 0x1b,0x4e,0x3f},
-{K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC, 
-	 K_SCAN,K_RSHSC,NC},
-{K_ASTER,NC,NC, K_ASTER,NC,NC, K_ASTER,NC,NC, 0x1b,0x4e,0x2a,0x1b,0x4e,0x2a},
-{K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC, 
-	 K_SCAN,K_ALTSC,NC},
-{K_SPACE,NC,NC, K_SPACE,NC,NC, K_NUL,NC,NC, K_SPACE,NC,NC, K_SPACE,NC,NC},
-{K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC, 
-	 K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC},
-{K_F1, K_F1S, K_F1, K_F1, K_F1S},
-{K_F2, K_F2S, K_F2, K_F2, K_F2S},
-{K_F3, K_F3S, K_F3, K_F3, K_F3S},
-{K_F4, K_F4S, K_F4, K_F4, K_F4S},
-{K_F5, K_F5S, K_F5, K_F5, K_F5S},
-{K_F6, K_F6S, K_F6, K_F6, K_F6S},
-{K_F7, K_F7S, K_F7, K_F7, K_F7S},
-{K_F8, K_F8S, K_F8, K_F8, K_F8S},
-{K_F9, K_F9S, K_F9, K_F9, K_F9S},
-{K_F10, K_F10S, K_F10, K_F10, K_F10S},
-{K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC, 
-	 K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC},
-{K_SCRL, K_NUL,NC,NC, K_SCRL, K_SCRL, K_NUL,NC,NC},
-{K_HOME, K_SEVEN,NC,NC, K_HOME, K_HOME, 0x1b,0x4e,0x37},
-{K_UA, K_EIGHT,NC,NC, K_UA, K_UA, 0x1b,0x4e,0x38},
-{K_PUP, K_NINE,NC,NC, K_PUP, K_PUP, 0x1b,0x4e,0x39},
-{0x1b,0x5b,0x53, K_MINUS,NC,NC, 0x1b,0x5b,0x53,0x1b,0x5b,0x53,0x1b,0x4e,0x2d},
-{K_LA, K_FOUR,NC,NC, K_LA, K_LA, 0x1b,0x4e,0x34},
-{0x1b,0x5b,0x47,K_FIVE,NC,NC,0x1b,0x5b,0x47, 0x1b,0x5b,0x47, 0x1b,0x4e,0x35},
-{K_RA, K_SIX,NC,NC, K_RA, K_RA, 0x1b,0x4e,0x36},
-{0x1b,0x5b,0x54,K_PLUS,NC,NC, 0x1b,0x5b,0x54, 0x1b,0x5b,0x54, 0x1b,0x4e,0x2b},
-{K_END, K_ONE,NC,NC, K_END, K_END, 0x1b,0x4e,0x31},
-{K_DA, K_TWO,NC,NC, K_DA, K_DA, 0x1b,0x4e,0x32},
-{K_PDN, K_THREE,NC,NC, K_PDN, K_PDN, 0x1b,0x4e,0x33},
-{K_INS, K_ZERO,NC,NC, K_INS, K_INS, 0x1b,0x4e,0x30},
-{K_DEL,NC,NC, K_PERIOD,NC,NC, K_DEL,NC,NC, K_DEL,NC,NC, 0x1b,0x4e,0x2e},
-{NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC},
-{NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC},
-{NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC,NC},
-{K_F11, K_F11S, K_F11, K_F11, K_F11S},
-{K_F12, K_F12S, K_F12, K_F12, K_F12S}
+ K_SCAN,K_LSHSC,NC},
+{K_BSLSH,NC,NC, K_PIPE,NC,NC,   K_FS,NC,NC,    0x1b,0x5c,NC,  0x1b,0x4e,0x7c},
+{K_z,NC,NC,     K_Z,NC,NC,      K_SUB,NC,NC,   0x1b,0x7a,NC,  0x1b,0x4e,0x5a},
+{K_x,NC,NC,     K_X,NC,NC,      K_CAN,NC,NC,   0x1b,0x78,NC,  0x1b,0x4e,0x58},
+{K_c,NC,NC,     K_C,NC,NC,      K_ETX,NC,NC,   0x1b,0x63,NC,  0x1b,0x4e,0x43},
+{K_v,NC,NC,     K_V,NC,NC,      K_SYN,NC,NC,   0x1b,0x76,NC,  0x1b,0x4e,0x56},
+{K_b,NC,NC,     K_B,NC,NC,      K_STX,NC,NC,   0x1b,0x62,NC,  0x1b,0x4e,0x42},
+{K_n,NC,NC,     K_N,NC,NC,      K_SO,NC,NC,    0x1b,0x6e,NC,  0x1b,0x4e,0x4e},
+{K_m,NC,NC,     K_M,NC,NC,      K_CR,NC,NC,    0x1b,0x6d,NC,  0x1b,0x4e,0x4d},
+{K_COMMA,NC,NC, K_LTHN,NC,NC,   K_COMMA,NC,NC, 0x1b,0x2c,NC,  0x1b,0x4e,0x3c},
+{K_PERIOD,NC,NC,K_GTHN,NC,NC,   K_PERIOD,NC,NC,0x1b,0x2e,NC,  0x1b,0x4e,0x3e},
+{K_SLASH,NC,NC, K_QUES,NC,NC,   K_SLASH,NC,NC, 0x1b,0x2f,NC,  0x1b,0x4e,0x3f},
+{K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC, K_SCAN,K_RSHSC,NC,
+     K_SCAN,K_RSHSC,NC},
+{K_ASTER,NC,NC, K_ASTER,NC,NC,  K_ASTER,NC,NC, 0x1b,0x4e,0x2a,0x1b,0x4e,0x2a},
+{K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC, K_SCAN,K_ALTSC,NC,
+     K_SCAN,K_ALTSC,NC},
+{K_SPACE,NC,NC, K_SPACE,NC,NC,  K_NUL,NC,NC,   0x1b,K_SPACE,NC, K_SPACE,NC,NC},
+{K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC,
+     K_SCAN,K_CLCKSC,NC, K_SCAN,K_CLCKSC,NC},
+{K_F1,  K_F1S,  K_F1,  K_F1A,  K_F1S},
+{K_F2,  K_F2S,  K_F2,  K_F2A,  K_F2S},
+{K_F3,  K_F3S,  K_F3,  K_F3A,  K_F3S},
+{K_F4,  K_F4S,  K_F4,  K_F4A,  K_F4S},
+{K_F5,  K_F5S,  K_F5,  K_F5A,  K_F5S},
+{K_F6,  K_F6S,  K_F6,  K_F6A,  K_F6S},
+{K_F7,  K_F7S,  K_F7,  K_F7A,  K_F7S},
+{K_F8,  K_F8S,  K_F8,  K_F8A,  K_F8S},
+{K_F9,  K_F9S,  K_F9,  K_F9A,  K_F9S},
+{K_F10, K_F10S, K_F10, K_F10A, K_F10S},
+{K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC,
+     K_SCAN,K_NLCKSC,NC, K_SCAN,K_NLCKSC,NC},
+{K_SCRL,         K_NUL,NC,NC,    K_SCRL,        K_SCRL,      K_NUL,NC,NC},
+{K_HOME,         K_SEVEN,NC,NC,  K_HOME,        K_HOME,      0x1b,0x4e,0x37},
+{K_UA,           K_EIGHT,NC,NC,  K_UA,          K_UA,        0x1b,0x4e,0x38},
+{K_PUP,          K_NINE,NC,NC,   K_PUP,         K_PUP,       0x1b,0x4e,0x39},
+{0x1b,0x5b,0x53, K_MINUS,NC,NC,  0x1b,0x5b,0x53, 0x1b,0x5b,0x53,
+     0x1b,0x4e,0x2d},
+{K_LA,           K_FOUR,NC,NC,   K_LA,          K_LA,        0x1b,0x4e,0x34},
+{0x1b,0x5b,0x47, K_FIVE,NC,NC,   0x1b,0x5b,0x47, 0x1b,0x5b,0x47,
+     0x1b,0x4e,0x35},
+{K_RA,           K_SIX,NC,NC,    K_RA,          K_RA,        0x1b,0x4e,0x36},
+{0x1b,0x5b,0x54, K_PLUS,NC,NC,   0x1b,0x5b,0x54, 0x1b,0x5b,0x54,
+     0x1b,0x4e,0x2b},
+{K_END,          K_ONE,NC,NC,    K_END,         K_END,       0x1b,0x4e,0x31},
+{K_DA,           K_TWO,NC,NC,    K_DA,          K_DA,        0x1b,0x4e,0x32},
+{K_PDN,          K_THREE,NC,NC,  K_PDN,         K_PDN,       0x1b,0x4e,0x33},
+{K_INS,          K_ZERO,NC,NC,   K_INS,         K_INS,       0x1b,0x4e,0x30},
+{0x1b,0x5b,0x39, K_PERIOD,NC,NC, K_DEL,NC,NC,   K_DEL,NC,NC, 0x1b,0x4e,0x2e},
+{NC,NC,NC,       NC,NC,NC,       NC,NC,NC,      NC,NC,NC,    NC,NC,NC},
+{NC,NC,NC,       NC,NC,NC,       NC,NC,NC,      NC,NC,NC,    NC,NC,NC},
+{NC,NC,NC,       NC,NC,NC,       NC,NC,NC,      NC,NC,NC,    NC,NC,NC},
+{K_F11,          K_F11S,         K_F11,         K_F11A,      K_F11S},
+{K_F12,          K_F12S,         K_F12,         K_F12A,      K_F12S}
 };
 
 
@@ -1065,12 +1083,15 @@ boolean_t	extended;
 	if ((!extended) && state != KS_NORMAL) {
 		if ((state&(KS_SHIFTED|KS_ALTED)) == (KS_SHIFTED|KS_ALTED))
 			state_idx = SHIFT_ALT;
+                /* CTRL should have higher priority than SHIFT.  That
+                   way, CTRL-SHIFT-2 and CTRL-2 produce the same keycode.
+                   --Derek Upham 1997/06/25 */
+		else if (state&KS_CTLED)
+			state_idx = CTRL_STATE;
 		else if (state&KS_SHIFTED)
 			state_idx = SHIFT_STATE;
 		else if (state&KS_ALTED)
 			state_idx = ALT_STATE;
-		else if (state&KS_CTLED)
-			state_idx = CTRL_STATE;
 	}
 
 	return (CHARIDX(state_idx));
