@@ -509,6 +509,21 @@ typedef u_char Scancode;
 #define K_F12		0x1b,0x4f,0x41
 #define K_F12S		0x1b,0x4f,0x61
 
+/* These are the Alt-FxxA #defines.  They work with the new keymap
+   -- Derek Upham 1997/06/25  */
+#define K_F1A		0x1b,0x4f,0x30
+#define K_F2A		0x1b,0x4f,0x31
+#define K_F3A		0x1b,0x4f,0x32
+#define K_F4A		0x1b,0x4f,0x33
+#define K_F5A		0x1b,0x4f,0x34
+#define K_F6A		0x1b,0x4f,0x35
+#define K_F7A		0x1b,0x4f,0x36
+#define K_F8A		0x1b,0x4f,0x37
+#define K_F9A		0x1b,0x4f,0x38
+#define K_F10A		0x1b,0x4f,0x39
+#define K_F11A		0x1b,0x4f,0x3a
+#define K_F12A		0x1b,0x4f,0x3b
+
 #define K_SCRL		0x1b,0x5b,0x4d
 #define K_HOME		0x1b,0x5b,0x48
 #define K_UA		0x1b,0x5b,0x41
